-- ------------------------------------------------------------------------------
-- RP Friends
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

-- Const: constants
--
-- available under RP_FRIENDS.const
--
local RP_FRIENDS = RP_FRIENDS;

table.insert(RP_FRIENDS.startup, function(state) -- startup function
  
  local GetAddOnMetadata = GetAddOnMetadata;
  
    -- ----------------- - Temporary values that are used later in rptags.globals
    --  
  local CFF               = "|cff";

  local BLACK             = "000000";
  local CYAN              = "33ff99";
  local GREEN             = "33ff33";
  local GREY              = "999999";
  local LIGHT_ORANGE      = "dd9933";
  local LITE_BLUE         = "00bbbb";
  local LITE_GREY         = "bbbbbb";
  local ORANGE            = "ff9966"; 
  local ORA_COLOR         = "bb00bb";
  local PINK              = "ff66ff";
  local RED               = "ff3333";
  local VIOLET            = "dd33aa";
  local WHITE             = "ffffff";
  local YELLOW            = "ffff66";
  local TEAL              = "33bb99";

  local RGBA_BN          = { r = 0.5, g = 0.5, b = 1.0, a = 1 };
  local RGBA_WHITE       = { r = 1,   g = 1,   b = 1,   a = 1 };
  local RGBA_YELLOW      = { r = 0.7, g = 0.7, b = 0,   a = 1 };
  
  local ICON_WIDTH       =  20;
  local SHORTTEXT_WIDTH  =  50;
  local TEXT_WIDTH       =  80;
  local LONGTEXT_WIDTH   = 110;
  local WIDETEXT_WIDTH   = 150;

  local RESET             = "|r";
  local MALE              = 2;
  local NEUTER            = 1;
  local FEMALE            = 3 ;
  local THEY              = 8675309; -- this is a custom number used internally
  local GENDER_UNKNOWN    = 5552368; -- also a custom number
  local GENDER_DEFAULT    = NEUTER;
  
  RP_FRIENDS.const = { 
    -- ------------- - addon details
    APP_ID           = "rpfriends",
    RP_COLOR         = CFF .. VIOLET,
    APP_COLOR        = CFF .. TEAL,
    ORAIBI           = CFF .. ORA_COLOR .. "Oraibi" .. RESET,
    VERSION          = GetAddOnMetadata(RP_FRIENDS.addOnName, "Version"),
    BACKDROP = {
      BLIZZTOOLTIP = { bgFile =  "Interface\\ChatFrame\\ChatFrameBackground",
                     edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
                     tileSize = 16, edgeSize = 16,
                     insets   = { left = 5, right = 5, top = 5, bottom = 4} }, 
      BLIZZTOOLTIPSEARCH = { -- bgFile =  "Interface\\ChatFrame\\ChatFrameBackground",
                     edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
                     tileSize = 16, edgeSize = 12,
                     -- insets   = { left = 2, right = 2, top = 2, bottom = 1} }, 
                     },
      BLIZZTOOLTIPMED = { bgFile =  "Interface\\ChatFrame\\ChatFrameBackground",
                     edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
                     tileSize = 16, edgeSize = 12,
                     insets   = { left = 2, right = 2, top = 2, bottom = 1} }, 
      BLIZZTOOLTIPTHIN = { bgFile =  "Interface\\ChatFrame\\ChatFrameBackground",
                     edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
                     tileSize = 16, edgeSize = 8,
                     insets   = { left = 2, right = 2, top = 2, bottom = 2} }, 
      ORIGINAL   = { bgFile =  "Interface\\ChatFrame\\ChatFrameBackground",
                       insets   = {top = -1, bottom = -1, left = -1, right = -1} },
      BAR = { bgFile = "Interface\\TargetingFrame\\UI-StatusBar", 
                    insets   = {top = 0, bottom = 0, left = 0, right = 0} },
      SKILLS = { bgFile = "Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar", 
                    insets   = {top = 0, bottom = 0, left = 0, right = 0} },
      BLANK = { bgFile =  "Interface\\ChatFrame\\ChatFrameBackground",
                    insets   = {top = 0, bottom = 0, left = 0, right = 0} },
      SHADED = { bgFile =  "Interface\\ChatFrame\\ChatFrameBackground",
                    insets   = {top = 0, bottom = 0, left = 0, right = 0} },
      SOLID = { bgFile = "Interface\\Buttons\\White8x8",
                    insets   = {top = 0, bottom = 0, left = 0, right = 0} },
      RAID = { bgFile = "Interface\\RaidFrame\\Raid-Bar-Hp-Fill",
                    insets   = {top = 0, bottom = 0, left = 0, right = 0} },
      }, -- statusbar
    STATUSBAR_ALPHA = { BLANK = 0, SHADED = 0.5, SKILLS = 0.75, BAR = 0.75, SOLID = 1, },
  ALIGN = {
    LEFT       = { H = "LEFT", V = "MIDDLE", }, CENTER = { H = "CENTER", V = "MIDDLE", }, RIGHT       = { H = "RIGHT", V = "MIDDLE", },
    TOPLEFT    = { H = "LEFT", V = "TOP",    }, TOP    = { H = "CENTER", V = "TOP",    }, TOPRIGHT    = { H = "RIGHT", V = "TOP",    },
    BOTTOMLEFT = { H = "LEFT", V = "BOTTOM", }, BOTTOM = { H = "CENTER", V = "BOTTOM", }, BOTTOMRIGHT = { H = "RIGHT", V = "BOTTOM", },
  }, 
  icons = {
    -- FAVORITE    = "COMMON\\FavoritesIcon",
    FAVORITE    = "PVPFrame\\PVP-BANNER-EMBLEM-4",
    DOWN_ARROW  = "|TInterface\\Buttons\\Arrow-Down-Down:0|t",
    T_          = "|TInterface\\",
    _t          = ":0|t",
    APP         = "Interface\\AddOns\\RP_Friends\\resources\\Graphics\\rpfriends-logo-256x256",
    APP_INLINE  = "|TInterface\\AddOns\\RP_Friends\\resources\\Graphics\\rpfriends-logo-transparent:20:40:0:-1:|t",
    BUTTON      =   "Interface\\AddOns\\RP_Friends\\resources\\Graphics\\rpfriends-person-2",
    -- BUTTON      =   "Interface\\AddOns\\RP_Friends\\resources\\Graphics\\rpfriends-person-64x64",
    -- BUTTON      =   "Interface\\AddOns\\RP_Friends\\resources\\Graphics\\rpfriends-person-32x32",
    COLORWHEEL  = "|TInterface\\OptionsFrame\\ColorblindSettings:0::0:0:256:256:0:148:0:148|t",
    ALLIANCE    = "ICONS\\UI_AllianceIcon-round",
    HORDE       = "ICONS\\UI_HordeIcon-round",
    -- ALLIANCE    = "|TInterface\\ICONS\\UI_AllianceIcon-round:0|t",
    -- HORDE       = "|TInterface\\ICONS\\UI_HordeIcon-round:0|t",
    TEST = "|TInterface\\HELPFRAME\\HelpIcon-Suggestion:0|t",
    -- ALLIANCE = "|TInterface\\FriendsFrame\\PlusManz-Alliance:0|t",
    -- HORDE = "|TInterface\\FriendsFrame\\PlusManz-Horde:0|t",
    PVE = "WorldMap\\UI-World-Icon",
    RP = "FriendsFrame\\UI-Toast-ChatInviteIcon",
    -- RP = "WorldMAp\\ChatBubble_64Grey",
    -- RP = "|TInterface\\WorldMAp\\ChatBubble_64Grey:0::0:0:64:64:0:64:0:64:0:236:0:0|t",
    -- ALLIANCE = "|TInterface\\Timer\\Alliance-Logo:0|t",
    -- HORDE = "|TInterface\\Timer\\Horde-Logo:0|t",
    STATUS = "RAIDFRAME\\UI-RaidFrame-Threat",
    NOTE = "FriendsFrame\\UI-FriendsFrame-Note",
    CURR = "COMMON\\mini-hourglass",
    DOWNTICK = "|TInterface\\Buttons\\Arrow-Down-Down:0::0:-6:64:64:0:64:0:64:0|t",
    UPTICK   = "|TInterface\\Buttons\\Arrow-Up-Down:0::0:0:64:64:0:64:0:64:0|t",
    PARAMS    = { 
                  DEFAULT   = "0::0:0:64:64:0:64:0:64"   ,
                  CROPPED   = "0::0:0:64:64:12:52:12:52" ,
                  FAVORITE  = "0::0:0:64:64:8:40:8:40" ,
                  HOURGLASS = "0::0:0:64:64:10:54:10:54" ,
                  MEDIAN    = "0::0:0:64:64:8:56:8:56"   ,
                  STATUS    = "0::0:0:64:64:4:60:4:60"   ,
                  MALE      = "0::0:0:128:64:0:64:0:64"  ,
                  FEMALE    = "0::0:0:128:64:64:128:0:64",
                  },
    CLIENTS = { 
      _ORDER = { "DISCONNECT", "APP", "BSAP", "WOW", "DST2", "VIPR", "S1", "PRO", "HERO", "WTCG", "S2", "D3", },
      WOW    = "Interface\\CHATFRAME\\UI-ChatIcon-WoW",
      S2     = "Interface\\CHATFRAME\\UI-ChatIcon-SC2",
      D3     = "Interface\\CHATFRAME\\UI-ChatIcon-D3",
      WTCG   = "Interface\\CHATFRAME\\UI-ChatIcon-WTCG",
      APP    = "Interface\\CHATFRAME\\UI-ChatIcon-Share",
      HERO   = "Interface\\CHATFRAME\\UI-ChatIcon-HotS",
      PRO    = "Interface\\CHATFRAME\\UI-ChatIcon-Overwatch",
      CLNT   = "Interface\\CHATFRAME\\UI-ChatIcon-Battlenet",
      S1     = "Interface\\CHATFRAME\\UI-ChatIcon-SC",
      DST2   = "Interface\\CHATFRAME\\UI-ChatIcon-Destiny2",
      VIPR   = "Interface\\CHATFRAME\\UI-ChatIcon-CallOfDutyBlackOps4",
      BSAP   = "Interface\\CHATFRAME\\UI-ChatIcon-Battlenet",
      -- DISCONNECT = "Interface\\CHARACTERFRAME\\Disconnect-Icon", -- too small
      -- DISCONNECT = "Interface\\COMMON\\Indicator-Red", -- bad rez
      DISCONNECT = "Interface\\Buttons\\UI-GroupLoot-Pass-Down", -- too light
      -- DISCONNECT = "Interface\\Buttons\\UI-GroupLoot-Pass-Up",  -- too bright
   },
  }, -- /icons
  COLOR_GREEN = GREEN,
  COLOR_RED   = RED,
  NBSP = "|TInterface\\Store\\ServicesAtlas:0::0:0:1024:1024:1023:1024:1023:1024|t",

  } -- Const

  return "const";
end);  -- startup function
