-- ------------------------------------------------------------------------------
-- rp:Friends
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

local RP_FRIENDS     = RP_FRIENDS;

table.insert(RP_FRIENDS.startup, function(state) -- startup function
  local Utils      = RP_FRIENDS.utils;
    
  if not RP_FRIENDS.cache then RP_FRIENDS.cache = {} end;
  if not Utils.config then Utils.config = {} end;
  if not Utils.help   then Utils.help   = {} end;

  local Config = Utils.config;
  local Const  = RP_FRIENDS.const;
  
  local loc         = Utils.locale.loc;
  local hi          = Utils.text.hilite;
  local notify      = Utils.text.notify;
  local version     = Utils.text.v(); -- yes, meant as a function call

  -- libraries
  --
  local PC_Dropdown = LibStub("PhanxConfig-Dropdown");
  local PC_Color    = LibStub("PhanxConfig-ColorPicker");
  local PC_Check    = LibStub("PhanxConfig-Checkbox");
  local PC_Text     = LibStub("PhanxConfig-EditBox");
  local PC_Slider   = LibStub("PhanxConfig-Slider");
  local LDBI        = LibStub("LibDBIcon-1.0");

  -- 
  local OptionsPanel = CreateFrame("Frame", "RP_FRIENDS_Main_OptionsPanel"); 
  local ColumnsPanel = CreateFrame("Frame", "RP_FRIENDS_Columns_OptionsPanel");

  local function checkBoxOnValueChanged(self, value)
    Config.set(self.setting, value) 
    ColumnsPanel.refresh(ColumnsPanel);
    RP_FRIENDS.displayFrame:Update("colsdatashowsize");
  end;

  local function openConfigMenu(menu)
    if     menu == "main"      then InterfaceOptionsFrame_OpenToCategory(RP_FRIENDS_Main_OptionsPanel);
    elseif menu == "columns"   then InterfaceOptionsFrame_OpenToCategory(RP_FRIENDS_Columns_OptionsPanel);
    elseif menu == "keybind"   then KeyBindingFrame_LoadUI(); KeyBindingFrame.mode = 1; ShowUIPanel(KeyBindingFrame);
    end;
  end;

  StaticPopupDialogs["RP_FRIENDS_OPEN_URL"] = {
    OnShow     =
      function(self, data)
        self.text:SetFormattedText(hi("RP_FRIENDS Link Window\n\nCopy the following URL for " .. data.name .. " and paste it into your browser, then close this window."));
        self.editBox:SetText(data.url);                        self.editBox:SetAutoFocus(true);
        self.editBox:HighlightText();                          self.editBox:SetWidth(300);
        self.button1:SetPoint("RIGHT", self, "RIGHT", -12, 0); self.text:SetJustifyH("LEFT");
        self.text:SetSpacing(3);
      end,
    text      = "",       wide       = true, closeButton  = true,
    button1   = "Got it", exclusive  = true, timeout      = 60,
    whileDead = true,     hasEditBox = true, OnAccept     = function(self) self:Hide() end,
    enterClicksFirstButton           = true, hideOnEscape = true,
    EditBoxOnEnterPressed            = function(self) self:GetParent():Hide() end,
    EditBoxOnEscapePressed           = function(self) self:GetParent():Hide() end,
    };

  local function openUrl(site, url, name)
        if site and loc("URL_" .. site:upper()) ~= "" then url = loc("URL_" .. site:upper()) end;
        StaticPopup_Show("RP_FRIENDS_OPEN_URL", nil, nil, { url = url, name = name, });
  end;


  -- main panel --------------------------------------------------------------------------------------------------------------------
        OptionsPanel.name = loc("OPT_MENU_MAIN");
        OptionsPanel.widgets = {};
        OptionsPanel.columnWidgets = {};
        OptionsPanel.colorWidgets = {};
        RP_FRIENDS.cache.configPanel = OptionsPanel;
        InterfaceOptions_AddCategory(OptionsPanel);

  local OptionsTitle = OptionsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        OptionsTitle:SetPoint("TOPLEFT", 16, -16);
        OptionsTitle:SetJustifyH("LEFT");
        OptionsTitle:SetText(loc("OPT_PAGE_MAIN"));
        OptionsPanel.title = OptionsTitle;
        OptionsPanel.version = OptionsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalTiny");
        OptionsPanel.version:SetPoint("TOPRIGHT", -16, -16);
        OptionsPanel.version:SetJustifyH("RIGHT");
        OptionsPanel.version:SetText(version);

        OptionsPanel.arrowLeft = CreateFrame('Button', nil, OptionsPanel);
        OptionsPanel.arrowLeft:SetNormalTexture("Interface\\ICONS\\misc_arrowleft");
        OptionsPanel.arrowLeft:SetSize(24, 24);
        OptionsPanel.arrowLeft:SetPoint("BOTTOMLEFT", OptionsPanel, "BOTTOMLEFT", 4, 4);

        OptionsPanel.arrowRight = CreateFrame('Button', nil, OptionsPanel);
        OptionsPanel.arrowRight:SetNormalTexture("Interface\\ICONS\\misc_arrowright");
        OptionsPanel.arrowRight:SetSize(24, 24);
        OptionsPanel.arrowRight:SetPoint("BOTTOMRIGHT", OptionsPanel, "BOTTOMRIGHT", -4, 4);

        OptionsPanel.arrowLeft.disable = OptionsPanel.arrowLeft:CreateTexture();
        OptionsPanel.arrowLeft.disable:SetTexture("Interface\\Icons\\misc_arrowleft");
        OptionsPanel.arrowLeft.disable:SetAllPoints();
        OptionsPanel.arrowLeft.disable:SetDesaturated(1);
        OptionsPanel.arrowLeft:SetNormalTexture(OptionsPanel.arrowLeft.disable);
        
        OptionsPanel.arrowRight:SetScript("OnClick", function(self) openConfigMenu("columns") end);
        OptionsPanel.arrowRight:SetScript("OnEnter", 
          function(self)
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
            GameTooltip:ClearLines();
            GameTooltip:AddLine("Columns")
            GameTooltip:Show();
          end);
        OptionsPanel.arrowRight:SetScript("OnLeave", function(self) GameTooltip:Hide(); end);

  local OptionsInstructionsFrame = CreateFrame('Frame', nil, OptionsPanel);
        OptionsInstructionsFrame:SetPoint("TOPLEFT", OptionsPanel, "TOPLEFT", 320, -36);
        OptionsInstructionsFrame:SetPoint("BOTTOMRIGHT", OptionsPanel, "BOTTOMRIGHT", -16, 40);
        OptionsInstructionsFrame:SetBackdrop(RP_FRIENDS.const.BACKDROP.BLIZZTOOLTIP);
        OptionsInstructionsFrame:SetBackdropColor(0.3, 0.3, 0.3, 0.3);
  local OptionsInstructions = CreateFrame('SimpleHTML', nil, OptionsInstructionsFrame);
        OptionsInstructions:SetPoint("TOPLEFT", OptionsInstructionsFrame, "TOPLEFT", 16, -16)
        OptionsInstructions:SetPoint("BOTTOMRIGHT", OptionsInstructionsFrame, "BOTTOMRIGHT", -16, 16);
        OptionsInstructions:SetJustifyH("LEFT");
        OptionsInstructions:SetWidth("260");
        OptionsInstructions:SetJustifyV("TOP");
        OptionsInstructions:SetFontObject("p", "GameFontNormal");
        OptionsInstructions:SetFontObject("h1", "GameFontHighlightLarge");
        OptionsInstructions:SetFontObject("h2", "GameFontHighlightHuge");
        OptionsInstructions:SetFontObject("h3", "GameFontNormalTiny");
        OptionsInstructions:SetText(hi(loc("OPT_DESC_MAIN"), true)); -- true = hilite for HTML

        OptionsInstructions:SetScript("OnHyperlinkClick",
          function(f, link, text, button, ...)
            local  protocol =  link:match("^(%a+):");
            local  dest     =  link:match(":(.+)");
            if     protocol == "config"                      then      openConfigMenu(dest);
            elseif protocol == "http" or protocol == "https" then openUrl(dest, link, text);
            end;
          end) -- function

        OptionsInstructions:SetHyperlinksEnabled(true);

        OptionsInstructions:SetScript("OnHyperlinkEnter",
          function(f, link, text, button, ...)
            local  protocol = link:match("^(%a+):");
            local  dest     = link:match(":(.+)");

            if     protocol == "http"   or
                   protocol == "https"  then   SetCursor("TAXI_CURSOR");
            elseif protocol == "config" then   SetCursor("INTERACT_CURSOR");
            end;

          end);

        OptionsInstructions:SetScript("OnHyperlinkLeave", function(f, link, text, button, ...) SetCursor(nil); end);


  local KeybindButton = CreateFrame('Button', nil, OptionsPanel, 'UIPanelButtonTemplate');
        -- KeybindButton:SetPoint("BOTTOMRIGHT", OptionsPanel, "BOTTOMRIGHT", -16, 16);
        KeybindButton:SetPoint("TOPRIGHT", OptionsPanel, "BOTTOMLEFT", -4, -14);
        KeybindButton:SetWidth(96);
        KeybindButton:SetText("Keybinds");
        KeybindButton:SetScript("OnEnter",
          function(self)
            GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT");
            GameTooltip:AddLine("Open Keybindings");
            GameTooltip:AddLine("Keybindings for " .. loc("APP_NAME") .. " are found in the AddOns section.", nil, nil, nil, true);
            GameTooltip:Show();
          end);
        KeybindButton:SetScript("OnLeave", function(self) GameTooltip:Hide() end);
        KeybindButton:SetScript("OnClick",
          function(self)
            InterfaceOptionsFrame:Hide()
            KeyBindingFrame_LoadUI(); KeyBindingFrame.mode = 1; ShowUIPanel(KeyBindingFrame);
          end);

  local ShowLoginMessage = PC_Check:New(OptionsPanel, "Show Login Message", "Check to show the login message") 
        ShowLoginMessage:SetPoint("TOPLEFT", OptionsTitle, "BOTTOMLEFT", 0, -16);
        ShowLoginMessage.setting = "LOGIN_MESSAGE";
        ShowLoginMessage.Text:SetWidth(160);
        OptionsPanel.widgets.ShowLoginMessage = ShowLoginMessage;
        ShowLoginMessage.OnValueChanged = function(self, value) Config.set(self.setting, value); OptionsPanel.widgets.ShowChangesMessage:SetEnabled(value) end;

  local ShowChangesMessage = PC_Check:New(OptionsPanel, "Show Changes Message", "Check to show the changes message"); 
        ShowChangesMessage:SetPoint("LEFT", ShowLoginMessage.Text, "LEFT", 0,  0);
        ShowChangesMessage:SetPoint("TOP",  ShowLoginMessage, "BOTTOM", 0,  0);
        ShowChangesMessage.setting = "CHANGES_MESSAGE";
        ShowChangesMessage.Text:SetWidth(160);
        ShowChangesMessage.OnValueChanged = function(self, value) Config.set(self.setting, value); end;
        OptionsPanel.widgets.ShowChangesMessage = ShowChangesMessage;

  local ShowMinimapButton = PC_Check:New(OptionsPanel, 'Show Minimap Button', 'Check to show the ' .. loc("APP_NAME") .. ' button on the minimap.');
        ShowMinimapButton:SetPoint("TOP", ShowChangesMessage, "BOTTOM", 0, 0);
        ShowMinimapButton:SetPoint("LEFT", ShowLoginMessage, "LEFT", 0, 0);
        ShowMinimapButton.setting = "BUTTON";
        ShowMinimapButton.Text:SetWidth(160);
        ShowMinimapButton.OnValueChanged = 
          function(self, value) Config.set(self.setting, value)
            if value then LDBI:Show("rp:Friends") else LDBI:Hide("rp:Friends") end;
            end;
        OptionsPanel.widgets.ShowMinimapButton = ShowMinimapButton;

  local ButtonBarOtherBlizzApps = PC_Check:New(OptionsPanel, loc("CONFIG_SHOW_OTHER_BLIZZ_GAMES"), hi(loc("CONFIG_SHOW_OTHER_BLIZZ_GAMES_TT")));
        ButtonBarOtherBlizzApps:SetPoint("TOPLEFT", ShowMinimapButton, "BOTTOMLEFT", 0, 0);
        ButtonBarOtherBlizzApps.setting = "SHOW_OTHER_BLIZZ_GAMES";
        ButtonBarOtherBlizzApps.Text:SetWidth(360);
        OptionsPanel.widgets.ButtonBarOtherBlizzApps = ButtonBarOtherBlizzApps;
        ButtonBarOtherBlizzApps.OnValueChanged = 
          function(self, value) 
            Config.set(self.setting, value); 
            if value then RP_FRIENDS.friendslist:SetMinResize(535, 165);
                          RP_FRIENDS.friendslist:SetWidth(math.max(535, RP_FRIENDS.friendslist:GetWidth()));
            else RP_FRIENDS.friendslist:SetMinResize(320, 165);
            end;
            RP_FRIENDS.friendslist:ShowButtonBar();
            RP_FRIENDS.displayFrame:Update("datashowoffsetcols");
          end;

  local ButtonOnBlizzFriendsList = PC_Check:New(OptionsPanel, 'rp:F Button on Friendslist', 'Show a button for ' .. loc("APP_NAME") .. " on the default Blizzard friendslist.");
        ButtonOnBlizzFriendsList:SetPoint("TOPLEFT", ButtonBarOtherBlizzApps, "BOTTOMLEFT", 0, 0);
        ButtonOnBlizzFriendsList.setting = "BUTTON_ON_FRIENDSLIST";
        ButtonOnBlizzFriendsList.Text:SetWidth(180)
        ButtonOnBlizzFriendsList.OnValueChanged =
          function(self, value)
            Config.set(self.setting, value);
            RP_FRIENDS_ButtonOnBlizzFriendsList:SetShown(value)
          end;
        OptionsPanel.widgets.ButtonOnBlizzFriendsList = ButtonOnBlizzFriendsList;
  
  local AppearanceHeading = OptionsPanel:CreateFontString(nil, 'OVERLAY', 'GameFontNormalLarge');
        AppearanceHeading:SetPoint("TOPLEFT", ButtonOnBlizzFriendsList, "BOTTOMLEFT", 0, -16);
        AppearanceHeading:SetJustifyH('LEFT');
        AppearanceHeading:SetJustifyV('TOP');
        AppearanceHeading:SetText('Friendslist Appearance');

  local AlphaSlider = PC_Slider:New(OptionsPanel, loc("CONFIG_ALPHA"), hi(loc("CONFIG_ALPHA_TT")), 0, 1, 0.05, true, true);
        AlphaSlider.setting = "ALPHA";
        AlphaSlider.OnValueChanged = function(self, value) 
          Config.set(self.setting, value); 
          local r, g, b = RP_FRIENDS.utils.color.hexaToNumber(Config.get("COLOR_BACKDROP"));
          RP_FRIENDS.friendslist:SetBackdropColor(r / 255, g / 255, b / 255, value);
          end;
        AlphaSlider:SetWidth(135);
        AlphaSlider:SetValue(Config.get("ALPHA"))
        AlphaSlider:SetPoint("TOPLEFT", AppearanceHeading, "BOTTOMLEFT", 0, -16);
        OptionsPanel.widgets.AlphaSlider = AlphaSlider;

  local FriendslistScale = PC_Slider:New(OptionsPanel, loc("CONFIG_SCALE"), hi(loc("CONFIG_SCALE_TT")), 0.25, 2, 0.05, true, true)
        FriendslistScale:SetPoint("LEFT", AlphaSlider, "RIGHT", 15, 0);
        FriendslistScale.setting = "SCALE";
        FriendslistScale:SetWidth(135);
        FriendslistScale.OnValueChanged =
          function(self, value)
            Config.set(self.setting, value);
            RP_FRIENDS.friendslist:SetScale(value)
          end; -- function
        FriendslistScale:SetValue(Config.get("SCALE"));
        OptionsPanel.widgets.FriendslistScale = FriendslistScale;

  local IconZoomSlider = PC_Slider:New(OptionsPanel, loc("CONFIG_ICON_ZOOM"), hi(loc("CONFIG_ICON_ZOOM_TT")), 1, 8, 0.5, false, true)
        IconZoomSlider:SetPoint("TOPLEFT", AlphaSlider, "BOTTOMLEFT", 0, -10);
        IconZoomSlider.setting = "ICON_ZOOM";
        IconZoomSlider:SetWidth(135);
        IconZoomSlider.OnValueChanged =
          function(self, value)
            Config.set(self.setting, value);
            RP_FRIENDS.displayFrame.popup:SetSize(32 * value, 32 * value)
          end;
        IconZoomSlider:SetValue(Config.get("ICON_ZOOM"));
        OptionsPanel.widgets.IconZoomSlider = IconZoomSlider;

  local ColorsHeading = OptionsPanel:CreateFontString(nil, 'OVERLAY', 'GameFontNormalLarge');
        ColorsHeading:SetPoint("TOP", IconZoomSlider, "BOTTOM", 0, -16);
        ColorsHeading:SetPoint("LEFT", AlphaSlider, "LEFT", 0, 0);
        ColorsHeading:SetJustifyH('LEFT');
        ColorsHeading:SetJustifyV('TOP');
        ColorsHeading:SetText(loc("H1_COLORS"));

  local BackdropColor = PC_Color:New(OptionsPanel, loc("CONFIG_COLOR_BACKDROP"), hi(loc("CONFIG_COLOR_BACKDROP_TT")));
        BackdropColor.setting = "COLOR_BACKDROP";
        BackdropColor:SetPoint("TOPLEFT", ColorsHeading, "BOTTOMLEFT", 0, -16)
        BackdropColor:SetWidth(135);
        OptionsPanel.colorWidgets.BackdropColor = BackdropColor;
        BackdropColor.OnValueChanged =
         function(self, r, g, b) 
           local color = RP_FRIENDS.utils.color.colorCode(r * 255, g * 255, b * 255); 
                 
                 color = color:gsub("^|cff", "");
                 Config.set(self.setting, color);
                 RP_FRIENDS.friendslist:SetBackdropColor(r, g, b, Config.get("ALPHA"));
         end; -- function

  local DefaultColor = PC_Color:New(OptionsPanel, loc("CONFIG_COLOR_DEFAULT"), hi(loc("CONFIG_COLOR_DEFAULT_TT")));
        DefaultColor.setting = "COLOR_DEFAULT";
        DefaultColor:SetPoint("LEFT", BackdropColor, "RIGHT", 15, 0)
        DefaultColor:SetWidth(135);
        OptionsPanel.colorWidgets.DefaultColor = DefaultColor;
        DefaultColor.OnValueChanged =
         function(self, r, g, b) 
           local color = RP_FRIENDS.utils.color.colorCode(r * 255, g * 255, b * 255); 
                 color = color:gsub("^|cff", "");
                 Config.set(self.setting, color);
                 RP_FRIENDS.displayFrame:Update("tintcolsshowdata");
                 for c, col in pairs(ColumnsPanel.columnWidgets) do  col.Text:SetTextColor(RP_FRIENDS.utils.color.hexaToFloat(Config.get("COLOR_" .. col.color))) end;
         end; -- function

  local RoleplayingColor = PC_Color:New(OptionsPanel, loc("CONFIG_COLOR_RP"), hi(loc("CONFIG_COLOR_RP_TT")));
        RoleplayingColor.setting = "COLOR_RP";
        RoleplayingColor:SetPoint("TOPLEFT", BackdropColor, "BOTTOMLEFT", 0, 0)
        RoleplayingColor:SetWidth(135);
        OptionsPanel.colorWidgets.RoleplayingColor = RoleplayingColor;
        RoleplayingColor.OnValueChanged =
         function(self, r, g, b) 
           local color = RP_FRIENDS.utils.color.colorCode(r * 255, g * 255, b * 255); 
                 
                 color = color:gsub("^|cff", "");
                 Config.set(self.setting, color);
                 RP_FRIENDS.displayFrame:Update("tintcolsshowdata");
                 for c, col in pairs(ColumnsPanel.columnWidgets) do  col.Text:SetTextColor(RP_FRIENDS.utils.color.hexaToFloat(Config.get("COLOR_" .. col.color))) end;
         end; -- function

  local BattleNetColor = PC_Color:New(OptionsPanel, loc("CONFIG_COLOR_BN"), hi(loc("CONFIG_COLOR_BN_TT")));
        BattleNetColor.setting = "COLOR_BN";
        BattleNetColor:SetPoint("LEFT", RoleplayingColor, "RIGHT", 15, 0)
        BattleNetColor:SetWidth(135);
        OptionsPanel.colorWidgets.BattleNetColor = BattleNetColor;
        BattleNetColor.OnValueChanged =
         function(self, r, g, b) 
           local color = RP_FRIENDS.utils.color.colorCode(r * 255, g * 255, b * 255); 
                 
                 color = color:gsub("^|cff", "");
                 Config.set(self.setting, color);
                 RP_FRIENDS.displayFrame:Update("tintcolsshowdata");
                 for c, col in pairs(ColumnsPanel.columnWidgets) do  col.Text:SetTextColor(RP_FRIENDS.utils.color.hexaToFloat(Config.get("COLOR_" .. col.color))) end;
         end; -- function

  -- -- columns panel -----------------------------------------------------------------------------------------------------------------------------------
        ColumnsPanel.name = " Columns";
        ColumnsPanel.parent = OptionsPanel.name;
        InterfaceOptions_AddCategory(ColumnsPanel);
        ColumnsPanel.widgets = {};
        ColumnsPanel.colorWidgets = {};
        ColumnsPanel.columnWidgets = {};

        ColumnsPanel.version = ColumnsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalTiny");
        ColumnsPanel.version:SetPoint("TOPRIGHT", -16, -16);
        ColumnsPanel.version:SetJustifyH("RIGHT");
        ColumnsPanel.version:SetText(version);

        ColumnsPanel.arrowLeft = CreateFrame('Button', nil, ColumnsPanel);
        ColumnsPanel.arrowLeft:SetNormalTexture("Interface\\ICONS\\misc_arrowleft");
        ColumnsPanel.arrowLeft:SetSize(24, 24);
        ColumnsPanel.arrowLeft:SetPoint("BOTTOMLEFT", ColumnsPanel, "BOTTOMLEFT", 4, 4);

        ColumnsPanel.arrowRight = CreateFrame('Button', nil, ColumnsPanel);
        ColumnsPanel.arrowRight:SetNormalTexture("Interface\\ICONS\\misc_arrowright");
        ColumnsPanel.arrowRight:SetSize(24, 24);
        ColumnsPanel.arrowRight:SetPoint("BOTTOMRIGHT", ColumnsPanel, "BOTTOMRIGHT", -4, 4);

        ColumnsPanel.arrowRight.disable = ColumnsPanel.arrowRight:CreateTexture();
        ColumnsPanel.arrowRight.disable:SetTexture("Interface\\Icons\\misc_arrowright");
        ColumnsPanel.arrowRight.disable:SetAllPoints();
        ColumnsPanel.arrowRight.disable:SetDesaturated(1);
        ColumnsPanel.arrowRight:SetNormalTexture(ColumnsPanel.arrowRight.disable);
        
        ColumnsPanel.arrowLeft:SetScript("OnClick", function(self) openConfigMenu("main") end);
        ColumnsPanel.arrowLeft:SetScript("OnEnter", 
          function(self)
            GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
            GameTooltip:ClearLines();
            GameTooltip:AddLine("Columns")
            GameTooltip:Show();
          end);
        ColumnsPanel.arrowLeft:SetScript("OnLeave", function(self) GameTooltip:Hide(); end);

  local ColumnsHeading = ColumnsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
        ColumnsHeading:SetPoint("TOPLEFT", ColumnsPanel, "TOPLEFT", 16, -16);
        ColumnsHeading:SetJustifyH("LEFT");
        ColumnsHeading:SetJustifyV("TOP");
        ColumnsHeading:SetText('Columns to Display');

  local anchor, lastElement, relAnchor, xOffset, yOffset = "TOPLEFT", ColumnsHeading, "BOTTOMLEFT", 0, -8;

  local colCount = 1;

  local function createColumnCheckbox(i, colData)
    local checkbox = PC_Check:New(ColumnsPanel, loc("CONFIG_SHOW_" .. colData.index), hi(loc("CONFIG_SHOW_" .. colData.index .. "_TT")));
          checkbox:SetPoint(anchor, lastElement, relAnchor, xOffset, yOffset);
          checkbox.setting = "SHOW_" .. colData.index;
          checkbox.color = colData.group;
          checkbox.colWidth = colData.width;
          checkbox.Text:SetWidth(120);
          checkbox.Text:SetTextColor(RP_FRIENDS.utils.color.hexaToFloat(Config.get("COLOR_" .. colData.group)));
          checkbox.OnValueChanged = checkBoxOnValueChanged;
     ColumnsPanel.columnWidgets["Show_" .. colData.index] = checkbox;
     if i % 4 == 0
     then lastElement = lastLine;      anchor = "TOPLEFT"; relAnchor = "BOTTOMLEFT"; xOffset = 0; yOffset = 0;
     else lastElement = checkbox.Text; anchor = "LEFT";    relAnchor = "RIGHT";      xOffset = 5; yOffset = 0;
          if i % 4 == 1 then lastLine = checkbox; end;
     end; -- if
     colCount = colCount + 1;
  end;
    
  local function createHeading(h1, cols)

    local heading = ColumnsPanel:CreateFontString(nil, 'OVERLAY', 'GameFontHighlightLarge');
    heading:SetPoint(anchor, lastElement, relAnchor, xOffset, yOffset);
    heading:SetPoint("LEFT", ColumnsPanel, "LEFT", 16, 0);

    heading:SetText(hi(loc(h1)));

    anchor, lastElement, relAnchor, xOffset, yOffset = "TOPLEFT", heading, "BOTTOMLEFT", 0, -4;
    for i=1, cols
    do createColumnCheckbox(i, RP_FRIENDS.grid.COLUMNS[colCount]) 
    end; -- for i
    anchor, lastElement, relAnchor, xOffset, yOffset = "TOP", lastElement, "BOTTOM", 0, -12;
  end;

  createHeading('H1_BATTLENET',  4);
  createHeading('H1_NAME',       5);
  createHeading('H1_STATUS',     3);
  createHeading('H1_SERVER',     3);
  createHeading('H1_FACTION',    2);
  createHeading('H1_RACE_CLASS', 6);
  createHeading('H1_MISC',       2);
  createHeading('H1_NOTES',      7);

  local KeybindButton2 = CreateFrame('Button', nil, ColumnsPanel, 'UIPanelButtonTemplate');
        -- KeybindButton2:SetPoint("BOTTOMRIGHT", OptionsPanel, "BOTTOMRIGHT", -16, 16);
        KeybindButton2:SetPoint("TOPRIGHT", ColumnsPanel, "BOTTOMLEFT", -4, -14);
        KeybindButton2:SetWidth(96);
        KeybindButton2:SetText("Keybinds");
        KeybindButton2:SetScript("OnEnter",
          function(self)
            GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT");
            GameTooltip:AddLine("Open Keybindings");
            GameTooltip:AddLine("Keybindings for " .. loc("APP_NAME") .. " are found in the AddOns section.", nil, nil, nil, true);
            GameTooltip:Show();
          end);
        KeybindButton2:SetScript("OnLeave", function(self) GameTooltip:Hide() end);
        KeybindButton2:SetScript("OnClick",
          function(self)
            InterfaceOptionsFrame:Hide()
            KeyBindingFrame_LoadUI(); KeyBindingFrame.mode = 1; ShowUIPanel(KeyBindingFrame);
          end);

  OptionsPanel.refresh = 
    function(self)
      -- update the current values
      for _, widget in pairs(self.widgets)       do widget:SetValue(Config.get(widget.setting)); end;
      for _, widget in pairs(self.columnWidgets) do widget:SetValue(Config.get(widget.setting)); end;
      for _, widget in pairs(self.colorWidgets)  do widget:SetValue( RP_FRIENDS.utils.color.hexaToFloat(Config.get(widget.setting))); end;
      self.widgets.ShowChangesMessage:SetEnabled(Config.get("LOGIN_MESSAGE"))
      self.colorWidgets.BattleNetColor:Disable();
    end;

  OptionsPanel.default =
    function(self)
      for _, widget in pairs(self.widgets) do RP_FRIENDS.utils.config.reset(widget.setting) end;
      for _, widget in pairs(self.columnWidgets) do RP_FRIENDS.utils.config.reset(widget.setting) end;
      for _, widget in pairs(self.colorWidgets) do RP_FRIENDS.utils.config.reset(widget.setting) end;
      RP_FRIENDS.displayFrame:Update("datashowncolsoffset", "Options Reset to Default")
      notify("OPTIONS_ARE_RESET");
    end;
      
  ColumnsPanel.refresh = 
    function(self)
      -- update the current values
      for _, widget in pairs(self.widgets)       do widget:SetValue(Config.get(widget.setting)) end;
      for _, widget in pairs(self.columnWidgets) do widget:SetValue(Config.get(widget.setting)); end;
      for _, widget in pairs(self.colorWidgets)  do local r, g, b = RP_FRIENDS.utils.color.hexaToNumber(Config.get(widget.setting)); 
                                                    widget:SetValue(r / 255, g / 255, b / 255, 1); end;
    end;

  ColumnsPanel.default =
    function(self)
      for _, widget in pairs(self.widgets) do RP_FRIENDS.utils.config.reset(widget.setting) end;
      for _, widget in pairs(self.columnWidgets) do RP_FRIENDS.utils.config.reset(widget.setting) end;
      for _, widget in pairs(self.colorWidgets) do RP_FRIENDS.utils.config.reset(widget.setting) end;
      RP_FRIENDS.displayFrame:Update("datashowncolsoffset", "Options Reset to Default")
      notify("OPTIONS_ARE_RESET");
    end;

  return "options"
      
end); -- startup function


