-- ------------------------------------------------------------------------------
-- rp:Friends
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

-- local addOnName, RP_FRIENDS = ...;
-- RP_FRIENDS.addOnName = addOnName;
-- _G.RP_FRIENDS = RP_FRIENDS;

local RP_FRIENDS = RP_FRIENDS;

table.insert(RP_FRIENDS.startup, function(state)

  RP_FRIENDS.friendDB = { };  -- initialize

  -- libraries
  local Const = RP_FRIENDS.const;
  local Utils = RP_FRIENDS.utils;

  local LibRealmInfo   = LibStub("LibRealmInfo");
  local friendslist        = RP_FRIENDS.friendslist;

  if not Utils.data then Utils.data = {} end;

  local BTAG_FIELDS  = RP_FRIENDS.grid.FIELDS.BTAG;
  local ACCT_FIELDS  = RP_FRIENDS.grid.FIELDS.ACCT;
  local REALM_FIELDS = RP_FRIENDS.grid.FIELDS.REALM;
  
  friendslist.tracked = { };
  friendslist.tracked_reverse = { };

  friendslist.msp_fields = { 
    "CO",  -- rp_oocinfo
    "CU",  -- rp_curr
    "FC",  -- rp_status
    "IC",  -- rp_icon
    "NA",  -- rp_name
    "NI",  -- rp_nick
    "NT",  -- rp_title
    "PX",  -- rp_honorific
    "RA",  -- rp_race
    "RC",  -- rp_class
    -- "AE",  -- rp_eyes
    -- "AG",  -- rp_age
    -- "FR",  -- rp_xp
    -- "GC",  -- acct_class
    -- "GF",  -- acct_faction
    -- "GR",  -- acct_race
    -- "GS",  -- acct_gender
    -- "GU",  -- acct_characterGUID
    -- "TT",  -- rp_tooltip
    -- "VA",  -- rp_client
  };

  local function generateFriendID(data) if not data then return "Unknown" end;
    local friendID = (data.bnet_battleTag and (data.bnet_battleTag) ..  "-" ..  (data.acct_client or data.bnet_client or "DISCONNECT"))
                  or (data.acct_characterName) .. "-" ..(data.acct_realmName or ""):gsub(" ","");
    return friendID;
  end; 
    
  function friendslist:UpdateBNFriends()
    local numBNFriends, numBNFriendsOnline = BNGetNumFriends();
    self.statusBar.bn.count:SetText(numBNFriendsOnline .. "/" .. numBNFriends);
    self.numBNFriends = numBNFriends;
    self.numBNFriendsOnline = numBNFriendsOnline;
       
    -- go through each bnet friend and grab their data
    for f = 1, numBNFriends
    do  local bnData = { BNGetFriendInfo(f) };
        local friend = {}; 

        -- store the data in friend[]
        for i, field in ipairs(BTAG_FIELDS) do friend[field] = bnData[i] end;

        local friendAccounts = BNGetNumFriendGameAccounts(f);

        friend.bnet_friendID = f;
        -- are they on any accounts right now?
        if    friendAccounts > 0
              -- if so then go through each account that friend is on and grab their data
        then  friend.accountList = {};
              for a = 1, friendAccounts
              do  local acctData = { BNGetFriendGameAccountInfo(f, a) };
                  local account = {};
                  for field, value in pairs(friend) do account[field] = value end;
                  -- store the data in account[]
                  for i, field in ipairs(ACCT_FIELDS) do account[field] = acctData[i] end;
                  -- is the account using wow?
                  if   account.acct_client == "WoW" and account.acct_realmName and account.acct_realmName ~= ""
                       -- if so then we're going to lookup their realm info
                  then local realmData = { LibRealmInfo:GetRealmInfo(account.acct_realmName) };
                       -- and store the data in account
                       for i, field in pairs(REALM_FIELDS) do account[field] = realmData[i] end;
                  end;
                  account.unitID = account.acct_characterName .. "-" .. account.acct_realmName:gsub(" ","");
                  -- each account gets a separate entry
                  local friendID = generateFriendID(account);
                  table.insert(friend.accountList, account.acct_client or "Unknown")
                  if not RP_FRIENDS.friendDB[friendID] then RP_FRIENDS.friendDB[friendID] = {} end;
                  for k, v in pairs (account) do RP_FRIENDS.friendDB[friendID][k] = v; end;
              end; -- for a
            -- each bnet friend who isn't on an account gets a separate entry
        else friend.bnet_client = "DISCONNECT"; 
             local friendID = generateFriendID(account);
             if not RP_FRIENDS.friendDB[friendID] then RP_FRIENDS.friendDB[friendID] = {} end;
             for k, v in pairs (friend) do RP_FRIENDS.friendDB[friendID][k] = v; end;
        end; -- if friendAccounts > 0
        
    end; -- for f
  end; -- function
       
  function friendslist:UpdateServerFriends()
    local numFriends = C_FriendList.GetNumFriends()
    local numFriendsOnline = C_FriendList.GetNumOnlineFriends()

    self.statusBar.server.count:SetText(numFriendsOnline .. "/" .. numFriends);

    self.numFriends = numFriends;
    self.numFriendsOnline = numFriendsOnline;
    self.myServer = GetRealmName();

    for f = 1, numFriends
    do  local friendInfo = C_FriendList.GetFriendInfoByIndex(f);
        local friend = {
          acct_characterName  = friendInfo.name, 
          acct_client         = (friendInfo.connected and "WoW") or "DISCONNECT", 
          acct_class          = friendInfo.className,
          acct_zoneName       = friendInfo.area,
          acct_charNoteText   = friendInfo.notes,
          acct_characterGUID  = friendInfo.guid,
          acct_level          = friendInfo.level,
          bnet_isDND          = friendInfo.dnd,
          bnet_isAFK          = friendInfo.afk,
          bnet_isReferAFriend = friendInfo.referAFriend,
          bnet_isOnline       = friendInfo.connected,
        }; 
       friend.acct_friendID = f;
       friend.acct_faction = UnitFactionGroup('player');
       friend.acct_realmName = self.myServer; 
       friend.unitID = friend.acct_characterName .. "-" .. friend.acct_realmName:gsub(" ","");
       local realmData = { LibRealmInfo:GetRealmInfo(self.myServer) };
       for i, field in pairs(REALM_FIELDS) do friend[field] = realmData[i] end;
       local friendID = generateFriendID(friend)
       if not RP_FRIENDS.friendDB[friendID] then RP_FRIENDS.friendDB[friendID] = {} end;
       for k, v in pairs (friend) do RP_FRIENDS.friendDB[friendID][k] = v; end;
    end;
  end;

  function friendslist:UpdatePlayerMSP(friendID)
    if not friendID or not RP_FRIENDS.friendDB[friendID] then return nil end;
    local unitID = RP_FRIENDS.friendDB[friendID].unitID;

    local profile = msp.char[unitID].field;
    local rpData = 
      { rp_age     = profile.AG, rp_birthplace = profile.BH, rp_class     = profile.RC,  rp_client  = profile.VA,
        rp_curr    = profile.CU, rp_desc       = profile.DE, rp_eyes      = profile.AE,  rp_glance  = profile.PE,
        rp_height  = profile.AH, rp_home       = profile.HH, rp_honorific = profile.PX,  rp_icon    = profile.IC,
        rp_motto   = profile.MO, rp_name       = profile.NA, rp_nick      = profile.NI, 
        rp_oocinfo = profile.CO, rp_title      = profile.NT, rp_race      = profile.RA,  rp_rstatus = profile.RS,
        rp_sliders = profile.PS, rp_status     = profile.FC, rp_tooltip   = profile.TT,  rp_weight  = profile.AW,
        rp_xp      = profile.FR, acct_gender   = profile.GS,
        };
    if state.mrp
    then local player, realm = unpack(RP_FRIENDS.utils.text.split(unitID:upper(), "-"));
         rpData.rp_noteText = mrpNotes 
                          and mrpNotes[realm]
                          and mrpNotes[realm][player]
    elseif state.trp3
      then rpData.rp_noteText = TRP3_API.profile.getPlayerCurrentProfile().notes
                            and TRP3_Register.character[unitID]
                            and TRP3_Register.character[unitID].profileID
                            and TRP3_API.profile.getPlayerCurrentProfile().notes[TRP3_Register.character[unitID].profileID]

    end;

    for k, v in pairs(rpData) do RP_FRIENDS.friendDB[friendID][k] = v end;
  end; -- function

  function friendslist:Track(friendID) if not friendID then return nil end; 
    local unitID = RP_FRIENDS.friendDB[friendID].unitID; 
    self.tracked[friendID] = unitID;
    self.tracked_reverse[unitID] = friendID;
    return true; 
  end;

  function friendslist:UnitIDIsTracked(unitID) if not unitID   then return nil end; return self.tracked_reverse[unitID]       end;
  function friendslist:IsTracked(friendID)     if not friendID then return nil end; return self.tracked[friendID];            end;
  function friendslist:Untrack(friendID)       if not friendID then return nil end; self.tracked[friendID] = nil; return nil; end;

  function friendslist:TrackAll()
    self.tracked = { }; -- reset the tracked list
    self.tracked_reverse = { };
    for friendID, f in pairs(RP_FRIENDS.friendDB)
    do  
        if   f.unitID and (f.acct_client == "WoW" or (not f.acct_client and f.bnet_client == "WoW"))
        then self.tracked[friendID]         = f.unitID; 
             self.tracked_reverse[f.unitID] = friendID;
        end;
                    
    end;
  end;

  function friendslist:RequestTrackedMSP()        for _, unitID in pairs(self.tracked) do  msp:Request(unitID, self.msp_fields) end; end;
  function friendslist:RequestMSPByUnitID(unitID) msp:Request(unitID, self.msp_fields) end;
  function friendslist:RequestMSP(friendID)       msp:Request(RP_FRIENDS.friendDB[friendID].unitID, self.msp_fields); end;

  function friendslist:UpdateRPNotes()
    for friendID, unitID in pairs(self.tracked)
    do local friendData = RP_FRIENDS.friendDB[friendID];

       if state.mrp
       then   local player, realm = unpack(RP_FRIENDS.utils.text.split(unitID:upper(), "-"));
              RP_FRIENDS.friendDB[friendID].rp_noteText = mrpNotes and mrpNotes[realm] and mrpNotes[realm][player]
       elseif state.trp3
       then   RP_FRIENDS.friendDB[friendID].rp_noteText = TRP3_API.profile.getPlayerCurrentProfile().notes
                          and TRP3_Register.character[unitID]
                          and TRP3_Register.character[unitID].profileID
                          and TRP3_API.profile.getPlayerCurrentProfile().notes[TRP3_Register.character[unitID].profileID]
       end; -- if stat

    end; -- for friendID
  end
  return "utils-data"
end);
